package cv08.server;

import cv08.server.action.Action;

import java.util.LinkedList;
import java.util.Queue;

/**
 * Hlavni vlakno serveru zpracovavajici vsechny prijate akce
 *
 * @author Karel Cemus
 */
public class Worker implements Runnable {

    private static final Queue<Action> actions = new LinkedList<Action>();

    public static void addAction( Action action ) {
        synchronized ( actions ) {
            actions.add( action );
            actions.notify();
        }
    }

    @Override
    public void run() {
        try {
            while ( true ) {

                if ( Thread.currentThread().isInterrupted() ) break;

                Action action;

                synchronized ( actions ) {

                    // pockej dokud neni pripravena nejaka akce
                    while ( actions.isEmpty() ) actions.wait();
                    // ziskej akci ke zpracovani
                    action = actions.poll();
                }

                // zpracuj akci mimo synchronized, uz nepracujeme v chranenem bloku
                action.execute();
            }
        } catch ( InterruptedException e ) {
            System.out.println( "Worker prerusen." );
        } finally {
            System.out.println( "Worker se ukoncuje." );
        }
    }
}
